"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("../config");
const throwError_1 = __importDefault(require("../utils/throwError"));
const info_1 = __importDefault(require("../utils/info"));
const getOverriddenValue_1 = __importDefault(require("../utils/getOverriddenValue"));
const IMPORT_GENERATE_URL = {
    defaultImport: "generateUrl",
    from: "route-codegen/generateUrl",
};
const IMPORT_REDIRECT_SERVER_SIDE_COMPONENT = {
    defaultImport: "RedirectServerSide",
    from: "route-codegen/RedirectServerSide",
};
const parseAppConfig = (appName, appConfig) => {
    const { routingType = config_1.RoutingType.Default, routes = {}, destinationDir, reactRouterV5LinkOptions, nextJSLinkOptions, defaultLinkOptions, generateLinkComponent = true, generateRedirectComponent = true, generateUseParams = true, generateUseRedirect = true, } = appConfig;
    if (routingType !== config_1.RoutingType.NextJS && routingType !== config_1.RoutingType.ReactRouterV5 && routingType !== config_1.RoutingType.Default) {
        return throwError_1.default([appName, "routingType"], `Routing type of an app must be either "${config_1.RoutingType.NextJS}" or "${config_1.RoutingType.ReactRouterV5}" or "${config_1.RoutingType.Default}"`);
    }
    const topLevelGenerateOptions = {
        generateLinkComponent,
        generateRedirectComponent,
        generateUseParams,
        generateUseRedirect,
    };
    const parsedConfig = {
        routes,
        destinationDir,
        routingType,
        routeLinkOptions: {
            ReactRouterV5: prepareLinkOptionsReactRouterV5({
                appName,
                routeLinkOptions: reactRouterV5LinkOptions,
                topLevelGenerateOptions,
            }),
            NextJS: prepareLinkOptionsNextJS({ appName, routeLinkOptions: nextJSLinkOptions, topLevelGenerateOptions }),
            Default: prepareLinkOptionsDefault({ appName, routeLinkOptions: defaultLinkOptions, topLevelGenerateOptions }),
        },
        importGenerateUrl: IMPORT_GENERATE_URL,
        importRedirectServerSide: IMPORT_REDIRECT_SERVER_SIDE_COMPONENT,
    };
    return parsedConfig;
};
const prepareLinkOptionsReactRouterV5 = (params) => {
    const { appName, routeLinkOptions, topLevelGenerateOptions } = params;
    const defaultOptions = {
        importLink: {
            from: "react-router-dom",
            namedImports: [{ name: "LinkProps" }, { name: "Link" }],
        },
        linkComponent: "Link",
        linkProps: "LinkProps",
        hrefProp: "to",
        generateLinkComponent: topLevelGenerateOptions.generateLinkComponent,
        generateRedirectComponent: topLevelGenerateOptions.generateRedirectComponent,
        generateUseRedirect: topLevelGenerateOptions.generateUseRedirect,
        generateUseParams: topLevelGenerateOptions.generateUseParams,
    };
    if (!routeLinkOptions) {
        info_1.default([appName, "reactRouterV5LinkOptions"], "custom options not found... Using default");
        return Object.assign({}, defaultOptions);
    }
    const result = Object.assign(Object.assign({}, defaultOptions), { generateLinkComponent: getOverriddenValue_1.default(defaultOptions.generateLinkComponent, routeLinkOptions.generateLinkComponent), generateRedirectComponent: getOverriddenValue_1.default(defaultOptions.generateRedirectComponent, routeLinkOptions.generateRedirectComponent), generateUseParams: getOverriddenValue_1.default(defaultOptions.generateUseParams, routeLinkOptions.generateUseParams), generateUseRedirect: getOverriddenValue_1.default(defaultOptions.generateUseRedirect, routeLinkOptions.generateUseRedirect) });
    if (!routeLinkOptions.importCustomLink) {
        info_1.default([appName, "reactRouterV5LinkOptions", "importCustomLink"], "custom options not found... Using default");
        return Object.assign({}, result);
    }
    info_1.default([appName, "reactRouterV5LinkOptions"], "using custom link options");
    const { importLink, hrefProp, linkComponent, linkProps } = handleImportCustomLink(appName, "reactRouterV5LinkOptions", routeLinkOptions.importCustomLink);
    return Object.assign(Object.assign({}, result), { importLink,
        linkComponent,
        hrefProp,
        linkProps });
};
const prepareLinkOptionsNextJS = (params) => {
    const { appName, routeLinkOptions, topLevelGenerateOptions } = params;
    const defaultOptions = {
        importLink: {
            from: "next/link",
            defaultImport: "Link",
            namedImports: [{ name: "LinkProps" }],
        },
        linkComponent: "Link",
        linkProps: "LinkProps",
        hrefProp: "href",
        generateLinkComponent: topLevelGenerateOptions.generateLinkComponent,
        generateUseParams: topLevelGenerateOptions.generateUseParams,
    };
    if (!routeLinkOptions) {
        info_1.default([appName, "nextJSLinkOptions"], "custom options not found... Using default");
        return Object.assign({}, defaultOptions);
    }
    const result = Object.assign(Object.assign({}, defaultOptions), { generateLinkComponent: getOverriddenValue_1.default(defaultOptions.generateLinkComponent, routeLinkOptions.generateLinkComponent), generateUseParams: getOverriddenValue_1.default(defaultOptions.generateUseParams, routeLinkOptions.generateUseParams) });
    if (!routeLinkOptions.importCustomLink) {
        info_1.default([appName, "nextJSLinkOptions", "importCustomLink"], "custom options not found... Using default");
        return Object.assign({}, result);
    }
    info_1.default([appName, "nextJSLinkOptions"], "using custom link options");
    const { importLink, hrefProp, linkComponent, linkProps } = handleImportCustomLink(appName, "nextJSLinkOptions", routeLinkOptions.importCustomLink);
    return Object.assign(Object.assign({}, result), { importLink,
        hrefProp,
        linkComponent,
        linkProps });
};
const prepareLinkOptionsDefault = (params) => {
    const { appName, routeLinkOptions, topLevelGenerateOptions } = params;
    const defaultOptions = {
        hrefProp: "href",
        linkComponent: "a",
        inlineLinkProps: {
            template: `type LinkProps = Omit<React.DetailedHTMLProps<React.AnchorHTMLAttributes<HTMLAnchorElement>, HTMLAnchorElement>, 'href'>`,
            linkProps: "LinkProps",
        },
        generateLinkComponent: topLevelGenerateOptions.generateLinkComponent,
        generateUseRedirect: topLevelGenerateOptions.generateUseRedirect,
        generateRedirectComponent: topLevelGenerateOptions.generateRedirectComponent,
    };
    if (!routeLinkOptions) {
        info_1.default([appName, "defaultLinkOptions"], "custom options not found... Using default");
        return Object.assign({}, defaultOptions);
    }
    const result = Object.assign(Object.assign({}, defaultOptions), { generateLinkComponent: getOverriddenValue_1.default(defaultOptions.generateLinkComponent, routeLinkOptions.generateLinkComponent), generateRedirectComponent: getOverriddenValue_1.default(defaultOptions.generateRedirectComponent, routeLinkOptions.generateRedirectComponent), generateUseRedirect: getOverriddenValue_1.default(defaultOptions.generateUseRedirect, routeLinkOptions.generateUseRedirect) });
    if (!routeLinkOptions.importCustomLink) {
        info_1.default([appName, "defaultLinkOptions", "importCustomLink"], "custom options not found... Using default");
        return Object.assign({}, result);
    }
    info_1.default([appName, "defaultLinkOptions"], "using custom link options");
    const { importLink, hrefProp, linkComponent, linkProps } = handleImportCustomLink(appName, "defaultLinkOptions", routeLinkOptions.importCustomLink);
    return Object.assign(Object.assign({}, result), { inlineLinkProps: undefined, importLink,
        hrefProp,
        linkComponent,
        linkProps });
};
const handleImportCustomLink = (appName, linkOptionName, importCustomLink) => {
    const errorPath = [appName, linkOptionName, "importCustomLink"];
    const { from, componentDefaultImport, componentNamedImport, propsNamedImport, hrefProp } = importCustomLink;
    if (!from) {
        return throwError_1.default(errorPath, '"from" is required');
    }
    if (!componentNamedImport && !componentDefaultImport) {
        return throwError_1.default(errorPath, 'either "componentNamedImport" or "componentDefaultImport" is required');
    }
    if (!propsNamedImport) {
        return throwError_1.default(errorPath, '"propsNamedImport" is required');
    }
    if (!hrefProp) {
        return throwError_1.default(errorPath, '"hrefProp" is required');
    }
    if (componentNamedImport && componentDefaultImport) {
        info_1.default(errorPath, '"componentNamedImport" and "componentDefaultImport" are supplied. "componentDefaultImport" will be used');
    }
    const linkComponent = "Link";
    const finalImportCustomLink = componentNamedImport
        ? {
            from,
            namedImports: [{ name: propsNamedImport }, { name: componentNamedImport, importAs: linkComponent }],
        }
        : {
            from,
            defaultImport: linkComponent,
            namedImports: [{ name: propsNamedImport }],
        };
    return { importLink: finalImportCustomLink, hrefProp, linkComponent, linkProps: propsNamedImport };
};
exports.default = parseAppConfig;
//# sourceMappingURL=parseAppConfig.js.map